// Copyright 2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/v2/generator.proto

#include "google/cloud/dialogflow_es/internal/generators_connection_impl.h"
#include "google/cloud/dialogflow_es/internal/generators_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/pagination_range.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>
#include <utility>

namespace google {
namespace cloud {
namespace dialogflow_es_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
namespace {

std::unique_ptr<dialogflow_es::GeneratorsRetryPolicy> retry_policy(
    Options const& options) {
  return options.get<dialogflow_es::GeneratorsRetryPolicyOption>()->clone();
}

std::unique_ptr<BackoffPolicy> backoff_policy(Options const& options) {
  return options.get<dialogflow_es::GeneratorsBackoffPolicyOption>()->clone();
}

std::unique_ptr<dialogflow_es::GeneratorsConnectionIdempotencyPolicy>
idempotency_policy(Options const& options) {
  return options
      .get<dialogflow_es::GeneratorsConnectionIdempotencyPolicyOption>()
      ->clone();
}

}  // namespace

GeneratorsConnectionImpl::GeneratorsConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<dialogflow_es_internal::GeneratorsStub> stub,
    Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(std::move(options),
                                      GeneratorsConnection::options())) {}

StatusOr<google::cloud::dialogflow::v2::Generator>
GeneratorsConnectionImpl::CreateGenerator(
    google::cloud::dialogflow::v2::CreateGeneratorRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->CreateGenerator(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::dialogflow::v2::CreateGeneratorRequest const&
                 request) {
        return stub_->CreateGenerator(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::cloud::dialogflow::v2::Generator>
GeneratorsConnectionImpl::GetGenerator(
    google::cloud::dialogflow::v2::GetGeneratorRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->GetGenerator(request),
      [this](
          grpc::ClientContext& context, Options const& options,
          google::cloud::dialogflow::v2::GetGeneratorRequest const& request) {
        return stub_->GetGenerator(context, options, request);
      },
      *current, request, __func__);
}

StreamRange<google::cloud::dialogflow::v2::Generator>
GeneratorsConnectionImpl::ListGenerators(
    google::cloud::dialogflow::v2::ListGeneratorsRequest request) {
  request.clear_page_token();
  auto current = google::cloud::internal::SaveCurrentOptions();
  auto idempotency = idempotency_policy(*current)->ListGenerators(request);
  char const* function_name = __func__;
  return google::cloud::internal::MakePaginationRange<
      StreamRange<google::cloud::dialogflow::v2::Generator>>(
      current, std::move(request),
      [idempotency, function_name, stub = stub_,
       retry = std::shared_ptr<dialogflow_es::GeneratorsRetryPolicy>(
           retry_policy(*current)),
       backoff = std::shared_ptr<BackoffPolicy>(backoff_policy(*current))](
          Options const& options,
          google::cloud::dialogflow::v2::ListGeneratorsRequest const& r) {
        return google::cloud::internal::RetryLoop(
            retry->clone(), backoff->clone(), idempotency,
            [stub](grpc::ClientContext& context, Options const& options,
                   google::cloud::dialogflow::v2::ListGeneratorsRequest const&
                       request) {
              return stub->ListGenerators(context, options, request);
            },
            options, r, function_name);
      },
      [](google::cloud::dialogflow::v2::ListGeneratorsResponse r) {
        std::vector<google::cloud::dialogflow::v2::Generator> result(
            r.generators().size());
        auto& messages = *r.mutable_generators();
        std::move(messages.begin(), messages.end(), result.begin());
        return result;
      });
}

Status GeneratorsConnectionImpl::DeleteGenerator(
    google::cloud::dialogflow::v2::DeleteGeneratorRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->DeleteGenerator(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::dialogflow::v2::DeleteGeneratorRequest const&
                 request) {
        return stub_->DeleteGenerator(context, options, request);
      },
      *current, request, __func__);
}

StatusOr<google::cloud::dialogflow::v2::Generator>
GeneratorsConnectionImpl::UpdateGenerator(
    google::cloud::dialogflow::v2::UpdateGeneratorRequest const& request) {
  auto current = google::cloud::internal::SaveCurrentOptions();
  return google::cloud::internal::RetryLoop(
      retry_policy(*current), backoff_policy(*current),
      idempotency_policy(*current)->UpdateGenerator(request),
      [this](grpc::ClientContext& context, Options const& options,
             google::cloud::dialogflow::v2::UpdateGeneratorRequest const&
                 request) {
        return stub_->UpdateGenerator(context, options, request);
      },
      *current, request, __func__);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_es_internal
}  // namespace cloud
}  // namespace google
